/*
 * Copyright (C) 2007 INdT - Instituto Nokia de Tecnologia . All rights reserved.
 *
 * This file is part of Download Manager
 *
 * Contact: Leonardo Sobral Cunha <leonardo.cunha@openbossa.org>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Instituto Nokia de Tecnologia. All rights are reserved.
 * Copying, including reproducing, storing, adapting or translating, any
 * or all of this material requires the prior written consent of
 * Instituto Nokia de Tecnologia. This material also contains
 * confidential information which may not be disclosed to others without
 * the prior written consent of Instituto Nokia de Tecnologia.
 */

#ifndef __CNL_CACHE_ENTRY_H__
#define __CNL_CACHE_ENTRY_H__

#include <glib.h>
#include <glib-object.h>

#include "cnl-download-item.h"

#include <time.h>

G_BEGIN_DECLS

typedef enum {
    CNL_CACHE_ENTRY_STATE_NONE,
    CNL_CACHE_ENTRY_STATE_DOWNLOADING,
    CNL_CACHE_ENTRY_STATE_LOCAL,
    CNL_CACHE_ENTRY_STATE_INCOMPLETE,
    CNL_CACHE_ENTRY_STATE_REMOVED,
    CNL_CACHE_ENTRY_STATE_ERROR
} cnl_cache_entry_state_t;

typedef enum {
    CNL_CACHE_ENTRY_ERROR_UNKNOWN,
    CNL_CACHE_ENTRY_ERROR_INTERNAL,
    CNL_CACHE_ENTRY_ERROR_DOWNLOADING
} cnl_cache_entry_error_t;

#define CNL_CACHE_ENTRY_TYPE \
    (cnl_cache_entry_get_type ())
#define CNL_CACHE_ENTRY(o) \
    (G_TYPE_CHECK_INSTANCE_CAST ((o), CNL_CACHE_ENTRY_TYPE, CnlCacheEntry))
#define CNL_CACHE_ENTRY_CLASS(k) \
    (G_TYPE_CHECK_CLASS_CAST((k), CNL_CACHE_ENTRY_TYPE, CnlCacheEntryClass))
#define IS_CNL_CACHE_ENTRY(o) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((o), CNL_CACHE_ENTRY_TYPE))
#define IS_CNL_CACHE_ENTRY_CLASS(k) \
    (G_TYPE_CHECK_CLASS_TYPE ((k), CNL_CACHE_ENTRY_TYPE))
#define CNL_CACHE_ENTRY_GET_CLASS(o) \
    (G_TYPE_INSTANCE_GET_CLASS ((o), CNL_CACHE_ENTRY_TYPE, CnlCacheEntryClass))

typedef struct _CnlCacheEntry        CnlCacheEntry;
typedef struct _CnlCacheEntryClass   CnlCacheEntryClass;
typedef struct _CnlCacheEntryPrivate CnlCacheEntryPrivate;

struct _CnlCacheEntryClass {
    GObjectClass parent;
};

struct _CnlCacheEntry {
    GObject parent;

    gchar* filename;

    guint id;

    gchar* url;
    guint64 size;

    cnl_cache_entry_state_t state;
    cnl_cache_entry_error_t state_reason;

    time_t entry_date;

    CnlCacheEntryPrivate *priv;
};

GType               cnl_cache_entry_get_type (void);

CnlCacheEntry*      cnl_cache_entry_new (const gchar* url,
                                         const gchar *filename,
                                         cnl_cache_entry_state_t state);

gboolean            cnl_cache_entry_get_state (CnlCacheEntry* self,
                                               gint *state,
                                               gint *state_reason,
                                               GError **error);

CnlDownloadItem*    cnl_cache_entry_get_download_item (CnlCacheEntry* self);

gboolean            cnl_cache_entry_register_obj (CnlCacheEntry *self,
                                                  gchar **service_name,
                                                  gchar **object_path);

void                cnl_cache_entry_set_state_removed (CnlCacheEntry *self);

gboolean            cnl_cache_entry_close (CnlCacheEntry *self);


G_END_DECLS

#endif
