/*
 * Copyright (C) 2007 INdT - Instituto Nokia de Tecnologia . All rights reserved.
 *
 * This file is part of Download Manager
 *
 * Contact: Leonardo Sobral Cunha <leonardo.cunha@openbossa.org>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Instituto Nokia de Tecnologia. All rights are reserved.
 * Copying, including reproducing, storing, adapting or translating, any
 * or all of this material requires the prior written consent of
 * Instituto Nokia de Tecnologia. This material also contains
 * confidential information which may not be disclosed to others without
 * the prior written consent of Instituto Nokia de Tecnologia.
 */

#ifndef __CNL_CACHE_MANAGER_H__
#define __CNL_CACHE_MANAGER_H__

#include <glib.h>
#include <glib-object.h>

#include <libgnomevfs/gnome-vfs.h>

#include "cnl-cache-entry.h"

#include <time.h>

#define CACHE_DIR_NAME "canola-cache"

G_BEGIN_DECLS

typedef enum {
    CNL_CACHE_POLICY_NO_REMOVAL,
    CNL_CACHE_POLICY_LEAST_USED_REMOVAL
} cnl_cache_manager_policy_t;

typedef enum {
    CNL_CACHE_OK,
    CNL_CACHE_UNMOUNTING,
    CNL_CACHE_ERROR
} cnl_cache_manager_state_t;

typedef enum {
    CNL_CACHE_ERROR_UNKNOWN,
    CNL_CACHE_ERROR_INTERNAL,
    CNL_CACHE_ERROR_COULD_NOT_REMOVE,
    CNL_CACHE_ERROR_FILE_NOT_FOUND,
    CNL_CACHE_ERROR_NO_SPACE_AVAILABLE,
    CNL_CACHE_ERROR_UNAVAILABLE
} cnl_cache_manager_error_t;

#define CNL_CACHE_MANAGER_TYPE \
    (cnl_cache_manager_get_type ())
#define CNL_CACHE_MANAGER(o) \
    (G_TYPE_CHECK_INSTANCE_CAST ((o), CNL_CACHE_MANAGER_TYPE, CnlCacheManager))
#define CNL_CACHE_MANAGER_CLASS(k) \
    (G_TYPE_CHECK_CLASS_CAST((k), CNL_CACHE_MANAGER_TYPE, CnlCacheManagerClass))
#define IS_CNL_CACHE_MANAGER(o) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((o), CNL_CACHE_MANAGER_TYPE))
#define IS_CNL_CACHE_MANAGER_CLASS(k) \
    (G_TYPE_CHECK_CLASS_TYPE ((k), CNL_CACHE_MANAGER_TYPE))
#define CNL_CACHE_MANAGER_GET_CLASS(o) \
    (G_TYPE_INSTANCE_GET_CLASS ((o), CNL_CACHE_MANAGER_TYPE, CnlCacheManagerClass))

typedef struct _CnlCacheManager        CnlCacheManager;
typedef struct _CnlCacheManagerClass   CnlCacheManagerClass;
typedef struct _CnlCacheManagerPrivate CnlCacheManagerPrivate;

struct _CnlCacheManagerClass {
    GObjectClass parent;

    /* removal policy to be used */
    gboolean (*removal_policy_apply) (CnlCacheManager *self, guint64 to_remove);

    /* method to add/remove an url to the removal priority list */
    void (*removal_policy_update_entry) (GList **removal_priority_list,
                                      const gchar *url);
    void (*removal_policy_remove_entry) (GList **removal_priority_list,
                                         const gchar *url);
};

struct _CnlCacheManager {
    GObject parent;

    cnl_cache_manager_state_t state;
    cnl_cache_manager_error_t state_reason;

    GnomeVFSVolumeMonitor *monitor;

    cnl_cache_manager_policy_t used_policy;
    gchar* dir_path;
    guint64 max_size;
    guint64 free_size;

    guint max_id;
};

GType               cnl_cache_manager_get_type (void);

CnlCacheManager*    cnl_cache_manager_new (gchar *base_path, guint64 cache_size);
gboolean            cnl_cache_manager_delete (CnlCacheManager *self);
gboolean            cnl_cache_manager_clean_all_entries (CnlCacheManager *self);

guint64             cnl_cache_manager_get_free_space (CnlCacheManager *self);
gboolean            cnl_cache_manager_check_free_space (CnlCacheManager *self,
                                                        guint64 file_size);

gboolean            cnl_cache_manager_contains_entry (CnlCacheManager *self,
                                                      const gchar *url);

CnlCacheEntry*      cnl_cache_manager_get_entry (CnlCacheManager *self,
                                                 const gchar *url);
GList*              cnl_cache_manager_get_entries (CnlCacheManager *self);

cnl_cache_manager_policy_t cnl_cache_manager_get_policy (CnlCacheManager *self);
void                cnl_cache_manager_set_policy (CnlCacheManager *self,
                                                  cnl_cache_manager_policy_t policy);

CnlCacheEntry*      cnl_cache_manager_add_entry (CnlCacheManager *self,
                                                 const gchar *url,
                                                 GError **error);

gboolean            cnl_cache_manager_remove_entry (CnlCacheManager *self,
                                                    const gchar *url);

gboolean            cnl_cache_manager_start_dbus_service (CnlCacheManager *self);

G_END_DECLS

#endif
