/*
 * Copyright (C) 2007 INdT - Instituto Nokia de Tecnologia . All rights reserved.
 *
 * This file is part of Download Manager
 *
 * Contact: Leonardo Sobral Cunha <leonardo.cunha@openbossa.org>
 *
 * This software, including documentation, is protected by copyright
 * controlled by Instituto Nokia de Tecnologia. All rights are reserved.
 * Copying, including reproducing, storing, adapting or translating, any
 * or all of this material requires the prior written consent of
 * Instituto Nokia de Tecnologia. This material also contains
 * confidential information which may not be disclosed to others without
 * the prior written consent of Instituto Nokia de Tecnologia.
 */

#ifndef __CNL_DOWNLOAD_ITEM_H__
#define __CNL_DOWNLOAD_ITEM_H__

#include <glib.h>
#include <glib-object.h>

#include <libgnomevfs/gnome-vfs.h>

G_BEGIN_DECLS

typedef enum {
    CNL_DOWNLOAD_STATUS_INITIAL,
    CNL_DOWNLOAD_STATUS_RUNNING,
    CNL_DOWNLOAD_STATUS_FINISHED,
    CNL_DOWNLOAD_STATUS_CANCELED,
    CNL_DOWNLOAD_STATUS_PAUSED,
    CNL_DOWNLOAD_STATUS_ERROR
} cnl_download_item_status_t;

typedef enum {
    CNL_DOWNLOAD_ERROR_UNKNOWN,
    CNL_DOWNLOAD_ERROR_INTERNAL,
    CNL_DOWNLOAD_ERROR_FILE_NOT_FOUND,
    CNL_DOWNLOAD_ERROR_BAD_FILE,
    CNL_DOWNLOAD_ERROR_INVALID_URI,
    CNL_DOWNLOAD_ERROR_ACCESS_DENIED,
    CNL_DOWNLOAD_ERROR_NO_SPACE_AVAILABLE,
    CNL_DOWNLOAD_ERROR_HOST_UNAVAILABLE,
    CNL_DOWNLOAD_ERROR_TIMEOUT,
    CNL_DOWNLOAD_ERROR_CANCELED,
    CNL_DOWNLOAD_ERROR_NO_NETWORK
} cnl_download_item_error_t;

#define CNL_DOWNLOAD_ITEM_TYPE \
    (cnl_download_item_get_type ())
#define CNL_DOWNLOAD_ITEM(o) \
    (G_TYPE_CHECK_INSTANCE_CAST ((o), CNL_DOWNLOAD_ITEM_TYPE, CnlDownloadItem))
#define CNL_DOWNLOAD_ITEM_CLASS(k) \
    (G_TYPE_CHECK_CLASS_CAST((k), CNL_DOWNLOAD_ITEM_TYPE, CnlDownloadItemClass))
#define IS_CNL_DOWNLOAD_ITEM(o) \
    (G_TYPE_CHECK_INSTANCE_TYPE ((o), CNL_DOWNLOAD_ITEM_TYPE))
#define IS_CNL_DOWNLOAD_ITEM_CLASS(k) \
    (G_TYPE_CHECK_CLASS_TYPE ((k), CNL_DOWNLOAD_ITEM_TYPE))
#define CNL_DOWNLOAD_ITEM_GET_CLASS(o) \
    (G_TYPE_INSTANCE_GET_CLASS ((o), CNL_DOWNLOAD_ITEM_TYPE, CnlDownloadItemClass))

typedef struct _CnlDownloadItem        CnlDownloadItem;
typedef struct _CnlDownloadItemClass   CnlDownloadItemClass;
typedef struct _CnlDownloadItemPrivate CnlDownloadItemPrivate;

struct _CnlDownloadItemClass {
    GObjectClass parent;
};

struct _CnlDownloadItem {
    GObject parent;

    gchar *url;
    gchar *dest_filename;

    cnl_download_item_status_t state;
    /* currently just used for errors */
    gint state_reason;
};

typedef guint64 (* CnlDownloadItemCheckSpaceCb) (CnlDownloadItem* handle,
                                                 gpointer user_data);

GType               cnl_download_item_get_type (void);

CnlDownloadItem*    cnl_download_item_new (const gchar* url,
                                           const gchar* dest_filename);

guint64     cnl_download_item_get_size (const gchar* url);

gboolean    cnl_download_item_start (CnlDownloadItem *self,
                                     gboolean resume,
                                     GError **error);

gboolean    cnl_download_item_cancel (CnlDownloadItem *self, GError **error);
gboolean    cnl_download_item_pause (CnlDownloadItem *self, GError **error);

gboolean    cnl_download_item_get_progress (CnlDownloadItem *self,
                                            cnl_download_item_status_t *status,
                                            guint64 *bytes_written,
                                            guint64 *size,
                                            GError **error);

void        cnl_download_item_set_check_space_cb (CnlDownloadItem *self,
                                                  CnlDownloadItemCheckSpaceCb check_space_cb,
                                                  gpointer user_data);

gboolean    cnl_download_item_get_state (CnlDownloadItem *self,
                                         gint *state,
                                         gint *state_reason,
                                         GError **error);

gboolean    cnl_download_item_register_obj (CnlDownloadItem *self,
                                            gchar **service_name,
                                            gchar **object_path);


G_END_DECLS

#endif
